unit copyhtmlunit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TfrmCopyHTML = class(TForm)
    btnCopy: TButton;
    edGridRef: TEdit;
    Label1: TLabel;
    procedure btnCopyClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmCopyHTML: TfrmCopyHTML;

implementation

{$R *.DFM}

{ ************************************************************************* }

function RightS(Txt: string; Size: Integer): string;
{ Returns <Size> chars from right of <Txt> }
begin
  if Size > Length(Txt) then
    Size := Length(Txt);
  Result := Copy(Txt, Length(Txt) - Size + 1, Size);
end;

{ ************************************************************************* }

function LeftS(Txt: string; Size: Integer): string;
{ Returns <Size> chars from left of <Txt> }
begin
  Result := Copy(Txt, 1, Size);
end;

{ ************************************************************************* }

function MidS(Txt: string; Pos, Size: Integer): string;
{ Returns <Size> chars from  <Txt>, starting at <Pos> }
begin
  Result := Copy(Txt, Pos, Size)
end;

{ ************************************************************************* }

function BeginS(Txt: string; Size: Integer): string;
{ Returns chars from beginning up to <Size> chars from end of <Txt> }
begin
  if Length(Txt) >= Size then
    Result := Copy(Txt, 1, (Length(Txt) - Size))
  else
    Result := '';
end;

{ ************************************************************************* }

function EndS(Txt: string; Posn: Integer): string;
{ Returns all chars from <Pos> to end of <Txt> }
begin
  if (Posn <= Length(Txt)) then
    Result := Copy(Txt, Posn, Length(Txt) - Posn + 1)
  else
    Result := '';
end;

{ ************************************************************************* }

function LRToOSGrid(Txt: string): string;
// Return an OS grid (X=12345&Y=45678) from a Landranger grid ref
var
  Lets, Nums: string;
  Mult: Double;
  LRE, LRN: Integer;
  E, N: Integer;
begin
  Result := '';
  Txt := UpperCase(Trim(Txt));
  if Length(Txt) < 2 then
    Exit;
  // Get letter part
  Lets := LeftS(Txt, 2);
  // Get number part
  Nums := Trim(EndS(Txt, 3));
  // Accept all formats from 'TL' to 'TL1234567890'
  case Length(Nums) of
    2: Mult := 10000;
    4: Mult := 1000;
    6: Mult := 100;
    8: Mult := 10;
    10: Mult := 1;
  else
    Mult := 0;
  end;
  // Calculate eastings and northings
  LRE := Round(StrToIntDef(LeftS(Nums, Length(Nums) div 2), 0) * Mult);
  LRN := Round(StrToIntDef(EndS(Nums, 1 + Length(Nums) div 2), 0) * Mult);
  N := (Pos(Lets[2], 'VWXYZQRSTULMNOPFGHJKABCDE') - 1) div 5;
  E := (Pos(Lets[2], 'AFLQVBGMRWCHNSXDJOTYEXPUZ') - 1) div 5;
  N := N + ((Pos(Lets[1], 'STUVWNOPQRHJKLMCDEFGXYZAB') - 1) div 5) * 5;
  E := E + ((Pos(Lets[1], 'SNHCXTOJDYUPKEZQLFAVRMGBW') - 1) div 5) * 5;
  N := N * 100000 + LRN;
  E := E * 100000 + LRE;
  Result := 'X=' + IntToStr(E) + '&Y=' + IntToStr(N);
end;

{ ************************************************************************* }

function UKStreetMap(LRGridRef: string; Zoom: Integer; Arrow: Boolean): string;
// Build a URL for the UK StreetMap website
// Pass Landranger grid ref, Zoom (1..7) and Arrow (True/False)
var
  A: Char;
begin
  if Arrow then
    A := 'Y'
  else
    A := 'N';
  Result := 'http://www.streetmap.co.uk/streetmap.dll?grid2map?'
    + LRtoOSGrid(LRGridRef) + '&Arrow=' + A + '&Zoom=' + IntToStr(Zoom);
end;

{ ************************************************************************* }

// Copy buffer to clipboard in specified format

procedure SetClipBuffer(Format: Integer; var Buffer; Size: Integer);
var
  Data: THandle;
  DataPtr: Pointer;
begin
  Data := GlobalAlloc(GMEM_MOVEABLE + GMEM_DDESHARE, Size);
  try
    DataPtr := GlobalLock(Data);
    try
      Move(Buffer, DataPtr^, Size);
      SetClipboardData(Format, Data);
    finally
      GlobalUnlock(Data);
    end;
  except
    GlobalFree(Data);
    raise;
  end;
end;

{ ************************************************************************* }

procedure CopyHTML(HTML: string);
// Copies a fully-tagged HTML string to the clipboard
// Tested with Frontpage Lite and Outlook Express
var
  StartH, EndH, StartF, EndF: Integer;
  Fragment: string;

  function MakeFragment: string;
  // Helper routine to build a properly-formatted HTML fragment
  begin
    Result := 'Version:0.9'#13#10;
    Result := Result + 'StartHTML:' + Format('%8.8d', [StartH]) + #13#10;
    Result := Result + 'EndHTML:' + Format('%8.8d', [EndH]) + #13#10;
    Result := Result + 'StartFragment:' + Format('%8.8d', [StartF]) + #13#10;
    Result := Result + 'EndFragment:' + Format('%8.8d', [EndF]) + #13#10;
    Result := Result + '<html><body>'#13#10;
    Result := Result + '<!--StartFragment-->'#13#10;
    Result := Result + Trim(HTML) + #13#10;
    Result := Result + '<!--EndFragment-->'#13#10;
    Result := Result + '</body></html>';
    StartH := Pos('<html>', Result) - 1;
    EndH := Length(Result);
    StartF := Pos('<!--S', Result) - 1;
    EndF := Pos('<!--E', Result) - 1;
  end;

// The actual copier
begin
 // Try to open clipboard
  if OpenClipboard(0) then
  begin
    try
     // Empty clipboard
      EmptyClipboard;
      // Build HTML fragment
      MakeFragment;
      // Return fragment with correct byte offsets embedded
      Fragment := MakeFragment;
      // Send fragment to clipboard
      SetClipBuffer(RegisterClipboardFormat('HTML Format'),
        Fragment[1], Length(Fragment) + 1);
    finally
     // Close clipboard
      CloseClipboard;
    end;
  end;
end;

{ ************************************************************************* }

procedure CopyURL(URL, URLText: string);
// Copy a URL and optional text to the clipboard
begin
 // Trim white space
  URL := Trim(URL);
  URLText := Trim(URLText);
  // Display a different string to the URL?
  if URLText = '' then
  // No, display the URL instead
    URLText := URL;
  CopyHTML('<a href="' + URL + '">' + URLText + '</a>');
end;

{ ************************************************************************* }

procedure CopyUKStreetMap(LRGridRef: string; Zoom: Integer; Arrow: Boolean);
// Generate a StreetMap URL for Landranger grid ref
// and copy it to the clipboard
begin
  CopyURL(UKStreetMap(LRGridRef, Zoom, Arrow), LRGridRef);
end;

{ ************************************************************************* }

procedure TfrmCopyHTML.btnCopyClick(Sender: TObject);
begin
  CopyUKStreetMap(edGridRef.Text, 3, True);
end;

{ ************************************************************************* }

end.

